#!/usr/bin/perl
#f:\perl\perl.exe

# Database position tracker, V1.3 by Dogfood
# Updated 10/6/97
# Email dog.food@poboxes.com
# Requires stats.db to be processed, stats.db must be sorted in the
# ranked player other

require "/usr/local/bin/frag-lib.pl";	# rawstats common functions

# === User variables ==========================================================
$trkstats = "/var/lib/qw/tracked.db";	# tracking database

# ============================================================================
# reads into each stat line
# store all information in memory
# important fields are kills, deaths, pos
sub read_stats {
  open(STATDB, "$statpath") or die "Can't open $statpath: $!\n";
  $linect = 1;
  while ($line = <STATDB>) {
    if (iscomment($line)) {	# ignore comments
      next;
    }
    chop $line;
    ($killer,$kills,$deaths,$su,$sc,$rk,$sk,$ef,$ma,$mb,$mc,$da,$db,$dc) = split(",",$line);
    $skey = sprintf("%06d",$linect);	# generate sort key (player position in table)
    $sortkey{$skey} = $killer;		# assign key
    $newpos{$killer} = $linect;		# store player pos
    $newdeaths{$killer} = $deaths;	# store deaths
    $newkills{$killer} = $kills;	# store kills
    $newothers{$killer} = "$su,$sc,$rk,$sk,$ef,$ma,$mb,$mc,$da,$db,$dc";
    $linect++;				# increment line counter
  }
  close STATDB;
}

# ============================================================================
# read the tracked database file, if it exists, other return
# compare stats.db and tracked.db in next function
# returns false if false to open tracked database
sub read_trk {
  open(TRKDB,"$trkstats") or return 0;
  while ($line = <TRKDB>) {
    if (!iscomment($line)) {
      chop $line;
      ($killer,$kills,$deaths,$prevpos,$ischeat,$lastchg) = split(",",$line);
      $status{$killer} = $lastchg;	# what happened last time
      $okills{$killer} = $kills;	# their last no. of kills
      $odeaths{$killer} = $deaths;	# their last no. of deaths
      $ocheat{$killer} = $change;	# if their have cheated before
      $oprevpos{$killer} = $prevpos;	# their last pos in tables
    }
  }
  close TRKDB;
  return 1;
}

# ============================================================================
# locates the cheaters
# then calculates the position changes, storing them in mem
# writes a tracked db file
# writes a new stats.db file
sub track_play {
  foreach $killer (sort keys %sortkey) {
    $id = $sortkey{$killer};
    $ocheat{$id} +=0;				# use field
    $status{$id} +=0;				# use field if not used
    $oprevpos{$id} +=0;				# use field if not used
    if ($ocheat{$id} != -1) {			# if not cheated, then check if cheated
      $dkills = $newkills{$id} - $oldkills{$id};
      $ddeaths = $newdeaths{$id} - $olddeaths{$id};
      if ($dkills<0 || $ddeaths<0) {		# if missing frags then
        print "Cheater found USERID=$id!\n";
        $ocheat{$id} = -1;			# mark as cheated
        $newdeaths{$id} = $olddeaths{$id};	# switch to higher deaths
        $newkills{$id} = $oldkills{$id};	# switch to high frags
      }
    }

    # if not cheated and have a previous position and not in the same place
    if (($ocheat{$id}!=-1) && ($oprevpos{$id}>0)) {
      if ($newpos{$id} != $oprevpos{$id}) {
        $status{$id} = $newpos{$id} - $oprevpos{$id};
        # status now +'ve/-'ve depending on change
      }
    }
#    print "$id $status{$id} $newpos{$id} $oprevpos{$id} $newkills{$id}\n";
  }

  # create a tracked database file for future updates in table ranking
  open TRKOUT, ">$trkstats" or die "Can't create tracking database $!\n";
  foreach $killer (sort keys %sortkey) {
    $id = $sortkey{$killer};
    print TRKOUT "$id,$newkills{$id},$newdeaths{$id},$newpos{$id},$ocheat{$id},$status{$id}\n";
  }
  close TRKOUT;
}

# =============================================================================
# create a new stats database with extra information for rankings page
sub write_nstats {
  open STATOUT, ">$statpath" or die "Can't create new stats database $!\n";
  foreach $idpos (sort keys %sortkey) {
    $id = $sortkey{$idpos};
    print STATOUT "$id,$newkills{$id},$newdeaths{$id},$newothers{$id},$status{$id},$oprevpos{$id}\n";
  }
  close STATOUT;
}

# =============================================================================
# Main

# === read in player id file ==================================================
open(PLAYIDDB, "$playiddb") or die "Can't open $playiddb: $!\n";
while ($line = <PLAYIDDB>) {
  chop $line;
  ($playid,$player) = split (/,/,$line);
  $id{$playid} = $player;
}
close PLAYIDDB;

read_stats;
read_trk;
track_play;
write_nstats;